@@ -45,6 +45,7 @@ static void getinstance(arena_state *s, const char *name)
   entry = symtab_stack_lookup(s, name);
   if (!entry || entry->type != SYMTAB_ENTRY_TMPL) {
     fatal(s, "use of undefined template `%s'", name);
+    return;
   }
   if (entry->entry_u.def.parent) {
     getinstance(s, entry->entry_u.def.parent);
@@ -72,6 +73,11 @@ value *eval_new(arena_state *s, expr *ex)
   s->new_sig  = NULL;
   getinstance(s, ex->name);
 
+  if (s->except_flag || s->exit_flag) {
+    symtab_stack_leave(s);
+    return value_make_void();
+  }
+
   res = value_make_struct();
   symtab_free(res->value_u.struct_val);
   res->value_u.struct_val = symtab_stack_pop(s);
@@ -93,6 +99,7 @@ value *eval_new(arena_state *s, expr *ex)
     temp = leave_struct(s, s->new_sig, ex->argc, ex->argv);
     if (!temp) {
       fatal(s, "no `this' at constructor `%s' exit", s->new_cons);
+      temp = value_make_void();
     }
     free_call_args(s, ex->argc, &argv);
     
@@ -116,11 +123,18 @@ value *eval_static(arena_state *s, expr *ex)
 
   symtab_stack_enter(s);
   getinstance(s, ex->tname);
+
+  if (s->except_flag || s->exit_flag) {
+    symtab_stack_leave(s);
+    return value_make_void();
+  }
   
   entry = symtab_stack_lookup(s, ex->name);
   if (!entry || entry->type != SYMTAB_ENTRY_FUNCTION ||
       !symtab_stack_local(s, ex->name)) {
     fatal(s, "call to undefined method `%s::%s'", ex->tname, ex->name);
+    symtab_stack_leave(s);
+    return value_make_void();
   }
   sig = call_sig_copy(entry->entry_u.sig);
   symtab_stack_leave(s);
@@ -155,8 +169,10 @@ value *eval_static_ref(arena_state *s, expr *ex)
   
   entry = symtab_lookup(sym, ex->name);
   if (!entry) {
-    fatal(s, "use of undefined template member `%s::%s'",
-      ex->tname, ex->name);
+    fatal(s, "use of undefined template member `%s::%s'", ex->tname,
+      ex->name);
+    symtab_free(sym);
+    return value_make_void();
   }
   if (entry->type == SYMTAB_ENTRY_VAR) {
     res = value_copy(entry->entry_u.var);
@@ -179,13 +195,16 @@ value *eval_method(arena_state *s, expr *ex)
   
   val = eval_expr(s, ex->inner);
   if (val->type != VALUE_TYPE_STRUCT) {
+    value_free(val);
     fatal(s, "method call on non-struct value");
+    return value_make_void();
   }
   
   entry = symtab_lookup(val->value_u.struct_val, ex->name);
   if (!entry || entry->type != SYMTAB_ENTRY_FUNCTION) {
     value_free(val);
     fatal(s, "call to undefined method `%s'", ex->name);
+    return value_make_void();
   }
 
   eval_call_args(s, ex->argc, ex->argv, &argv);
