@@ -11,7 +11,7 @@
 /*
  * Evaluate switch
  */
-void eval_stmt_switch(stmt *st, int cookie)
+void eval_stmt_switch(arena_state *s, stmt *st, int cookie)
 {
   value *val, *guard, *equal;
   stmt_list *list;
@@ -22,7 +22,7 @@ void eval_stmt_switch(stmt *st, int cookie)
   sanity(st && st->type == STMT_SWITCH && st->block);
   list = (stmt_list *) st->block;
 
-  val = eval_expr(st->expr);
+  val = eval_expr(s, st->expr);
   
   for (i = 0; i < list->len; i++) {
     label = list->list[i];
@@ -33,7 +33,7 @@ void eval_stmt_switch(stmt *st, int cookie)
     if (!go && label->type != STMT_CASE) continue;
 
     if (!go) {
-      guard = eval_expr(label->expr);
+      guard = eval_expr(s, label->expr);
       equal = eval_order_equal(val, guard);
       go = equal->value_u.bool_val;
       value_free(guard);
@@ -42,14 +42,14 @@ void eval_stmt_switch(stmt *st, int cookie)
     
     if (go) {
       handled = 1;
-      eval_stmt_list((stmt_list *) label->block, cookie);
+      eval_stmt_list(s, (stmt_list *) label->block, cookie);
     }
     
     if (go && !label->thru) break;
   }
 
   if (!handled && def >= 0) {
-    eval_stmt_list((stmt_list *) list->list[def]->block, cookie);
+    eval_stmt_list(s, (stmt_list *) list->list[def]->block, cookie);
   }
 
   value_free(val);
