@@ -2,6 +2,7 @@
  * Raw memory allocation interface
  */
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -161,6 +162,16 @@ value *mem_cnull(arena_state *s, unsigned int argc, value **argv)
   return value_make_resource(NULL, mem_cleanup);
 }
 
+/*
+ * Check for resource containing C NULL pointer
+ */
+value *mem_is_null(arena_state *s, unsigned int argc, value **argv)
+{
+  memblock *mem = RESDATA_OF(argv[0]);
+  
+  return value_make_bool(is_mem(argv[0]) && (!mem || !mem->data));
+}
+
 /*
  * Create resource from string value
  */
